/* jQuery Date Picker styles */

#ui-datepicker-div{
	padding:6px;
	background-color:#dfdfdf;
	border:1px solid #c0c0c0;
	display:none;
	}
.ui-datepicker-next, .ui-datepicker-prev{
	cursor: pointer;
	font-size:10px;
	font-weight: bold;
	padding:4px 6px;
	background-color:#3f3f3f;
	color:white;
	}
.ui-datepicker-next{
	float:right;
	}
.ui-datepicker-prev{
	float:left;
	}
.ui-datepicker-title{
	font-size:13px;
	font-weight: bold;
	padding:3px 0 10px;
	text-align: center;
	}

/* Table Styles */

.ui-datepicker-calendar{
	margin:0 auto;
	}
.ui-datepicker-calendar thead th{
	text-align: center;
	font-size:11px;
	padding:6px;
	}
.ui-state-default{
	background-color: #fefefe;
	border:2px solid #dfdfdf;
	display: block;
	font-size:11px;
	font-weight: bold;
	padding:6px;
	text-align: center;
	}
a.ui-state-hover{
	background-color:#0053A8;
	color:#fff;
	text-decoration: none;
	}
a.ui-state-active{
	background-color:#0053A8;
	color:#fff;
	}