/* Default style sheet for jQuery date picker v3.7.4. */
#datepick-div, .datepick-inline {
	font-family: Arial,Helvetica,sans-serif;
	font-size: 14px;
	padding: 0;
	margin: 0;
	background: #ddd;
	color: #000;
	width: 15em;
}
#datepick-div {
	display: none;
	border: 1px solid #777;
	z-index: 100; /*must have*/
}
.datepick-inline {
	float: left;
	display: block;
	border: 1px solid #777;
}
.datepick-rtl {
	direction: rtl;
}
.datepick-dialog {
	padding: 5px !important;
	border: 4px ridge #ddd !important;
}
.datepick-disabled {
	position: absolute;
	z-index: 100;
	background-color: white;
	opacity: 0.5;
	filter: alpha(opacity=50);
}
button.datepick-trigger {
	width: 25px;
	padding: 0px;
}
img.datepick-trigger {
	margin: 2px;
	vertical-align: middle;
}
.datepick-prompt {
	float: left;
	padding: 2px;
}
* html .datepick-prompt { /* IE6 */
	width: 15em;
}
.datepick-control, .datepick-links, .datepick-header, .datepick {
	clear: both;
	float: left;
	width: 100%;
	color: #d6fad6;
}
.datepick-control {
	background: #5555A0;
	padding: 2px 0px;
}
.datepick-links {
	background: #5555A0;
	padding: 2px 0px;
}
.datepick-control, .datepick-links {
	font-weight: bold;
	font-size: 80%;
}
.datepick-links label {
	padding: 2px 5px;
	color: #888;
}
.datepick-clear, .datepick-prev {
	float: left;
	width: 34%;
}
.datepick-rtl .datepick-clear, .datepick-rtl .datepick-prev {
	float: right;
	text-align: right;
}
.datepick-current {
	float: left;
	width: 30%;
	text-align: center;
}
.datepick-close, .datepick-next {
	float: right;
	width: 34%;
	text-align: right;
}
.datepick-rtl .datepick-close, .datepick-rtl .datepick-next {
	float: left;
	text-align: left;
}
.datepick-header {
	background: #5555A0;
	text-align: center;
	font-weight: bold;
	min-height: 1.75em;
}
.datepick-header select {
	background: white;
	color: #5555A0;
	border: 0px;
	font-weight: bold;
}
.datepick-header span {
	position: relative;
	top: 3px;
}
.datepick {
	background: #ccc;
	text-align: center;
	font-size: 100%;
}
.datepick a {
	display: block;
	width: 100%;
	color: #000;
}
.datepick-title-row {
	background: #257025;
}
.datepick-title-row th {
	text-align: center;
	font-weight: normal;
}
.datepick-days-row {
	background: #eee;
	color: #666;
}
.datepick-week-col {
	background: #777;
	color: #fff;
}
.datepick-days-cell {
	color: #000;
	border: 1px solid #ddd;
}
.datepick-other-month {
	background: #fff;
}
.datepick-week-end-cell {
	background: #ddd;
}
.datepick-title-row .datepick-week-end-cell {
	background: #257025;
}
.datepick-week-over {
	background: #ddd;
}
.datepick-unselectable {
	color: #888;
}
.datepick-today {
	background: #fcc;
}
.datepick-current-day, .datepick-current-day a {
	background: #777;
	color: #fff;
}
.datepick-days-cell-over {
	background: #fff;
	color: #000;
	border: 1px solid #777;
}
.datepick-status {
	background: #ddd;
	width: 100%;
	font-size: 80%;
	text-align: center;
}
#datepick-div a, .datepick-inline a {
	cursor: pointer;
	margin: 0;
	padding: 0;
	text-decoration: none;
}
.datepick-inline .datepick-links a {
	padding: 0 5px !important;
}
.datepick-control a, .datepick-links a {
	padding: 2px 5px !important;
	color: #eee;
}
.datepick-title-row a {
	color: #eee;
}
.datepick-control a:hover {
	background: white;
	color: #333;
}
.datepick-links a:hover, .datepick-title-row a:hover {
	background: white;
	color: #333;
}
.datepick-multi .datepick {
	border: 1px solid #777;
}
.datepick-one-month {
	float: left;
	width: 15em;
}
.datepick-new-row {
	clear: left;
}
/* ___________ IE6 IFRAME FIX ________ */
.datepick-cover {
    display: none; /*sorry for IE5*/
    display/**/: block; /*sorry for IE5*/
    position: absolute; /*must have*/
    z-index: -1; /*must have*/
    filter: mask(); /*must have*/
    top: -1px; /*must have*/
    left: -1px; /*must have*/
    width: 187px; /*must have*/
    height: 187px; /*must have*/
}
